/*
 * $Id: kl_array.h,v 1.3 2000/10/06 00:08:19 laik Exp $
 *
 * Dynamic KLArrays
 *
 * Copyright (C) 1993 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ARRAY_H__
#define __ARRAY_H__

#include <stdarg.h>
#include "kl_util.h"

struct KLArray;
typedef struct KLArray KLArray;

/* Create/delete/verify arrays. */
struct KLArray *
KLArrayNew(void);
void
KLArrayFree(KLArray *array, int free_buffer);
void
KLArrayVerify(KLArray *array);

/* Read from array. */
size_t
KLArrayGetLen(const KLArray *array);
void *
KLArrayGetBuffer(KLArray *array, size_t *len);
void *
KLArrayGetSection(KLArray *array, size_t begin, size_t end);

/* Write to array. */
void
KLArrayInsert(KLArray *array, size_t begin, void *buf, size_t len);
void
KLArrayInsertCopy(KLArray *array, size_t begin, const void *buf, size_t len);
void
KLArrayInsertVSprintf(KLArray *array, size_t begin, const char *format, 
		    va_list args);
void
KLArrayInsertSprintf(KLArray *array, size_t begin, const char *format, 
		   ...) __attribute__ ((format (printf, 3, 4)));
void
KLArrayVStrCat(KLArray *array, const char *format, va_list args);
void
KLArrayStrCat(KLArray *array, const char *format, 
	    ...) __attribute__ ((format (printf, 2, 3)));

/* Delete from the array. */
void
KLArrayDelete(KLArray *array, size_t begin, size_t end);

/* Test array functionality. */
void
KLArrayTest(void);

#endif
