/*
 * $Id: kl_event.h,v 1.5 2001/06/20 01:35:43 laik Exp $
 *
 * This file must be included in order to use the event routines.
 *
 * Copyright (C) 1999 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __EVENT_H__
#define __EVENT_H__

#include <time.h>
#include "kl_exception.h"

struct KLEventManager;
typedef struct KLEventManager KLEventManager;

struct KLEvent;
typedef struct KLEvent KLEvent;

struct KLEventDefRegistration;
typedef struct KLEventDefRegistration KLEventDefRegistration;

typedef enum {
    KLEventTypeFDRead,
    KLEventTypeFDWrite,
    KLEventTypeFDExcept,
    KLEventTypeTimer,
    KLEventTypeSignal,
    KLEventTypeClient
} KLEventType;

#define KLEventTypeLast (KLEventTypeClient + 1)

typedef KLException * (*KLEventHandlerFun)(const KLEvent *event);

typedef void (*KLEventClientDataDeleteFun)(void *client_data);

typedef union KLEventTypeData {
    int fd;
    int signo;
    struct timespec timer_interval;   /* Timeout this often. */
    unsigned int client_id;
} KLEventTypeData;

typedef struct KLEventDef {
    KLEventHandlerFun handler_fun;
    void *client_data;
    KLEventClientDataDeleteFun client_data_delete_fun;
    KLEventType type;
    KLEventTypeData type_data;
} KLEventDef;

struct KLEvent {
    struct timespec time;              /* Desired or actual event time. */
    const struct KLEventDef *def;
};

typedef struct KLEventManagerDef {
    int read_log_fd;
    int write_log_fd;
} KLEventManagerDef;

KLException *
KLEventManagerNew(KLEventManagerDef *manager_def, KLEventManager **manager_p);

void
KLEventManagerDelete(KLEventManager *event_manager);

KLEventDefRegistration *
KLEventDefRegister(KLEventManager *manager, KLEventDef *def);

void
KLEventDefUnregister(
    KLEventManager *manager, KLEventDefRegistration *Registration);

KLException *
KLEventDoNext(
    KLEventManager *event_manager, int num_events, struct timespec *timeout);

void
KLEventAppend(
    KLEventManager *manager, KLEventType type, KLEventTypeData *type_data);

void
KLEventTest(void);

#endif
