/*
 * $Id: kl_interval.h,v 1.4 2000/11/05 17:56:00 laik Exp $
 *
 * This file contains interfaces for routines which manage intervals.
 * KLIntervals are defined by closed range between two floating point floating
 * point numbers.
 *
 * Copyright (C) 1993 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __INTERVAL_H__
#define __INTERVAL_H__

#include "kl_array.h"
#include "kl_exception.h"

struct KLIntervalSet;
typedef struct KLIntervalSet KLIntervalSet;

struct KLInterval;
typedef struct KLInterval KLInterval;

typedef void (KLIntervalFreeFunc)(void *client_data, void *item);

/*
 * This is a function which is run on all intervals which
 * intersect a certain range. It returns 0 if successful, non-zero
 * otherwise.
 */
typedef KLException * (KLIntervalIntersectFun)(
    KLInterval *interval, void *client_data);

struct KLIntervalSet *
KLIntervalSetNew(
    unsigned int dimensions, KLIntervalFreeFunc *free_func, void *client_data);

void
KLIntervalSetFree(KLIntervalSet *interval_set);

void
KLIntervalSetInitialize(KLIntervalSet *interval_set);

void
KLIntervalMerge(
    KLIntervalSet *interval_set, const double *lower, const double *upper);

KLInterval *
KLIntervalNew(
    KLIntervalSet *interval_set, double *lower, double *upper,
    void *client_data);

void
KLIntervalFree(KLIntervalSet *interval_set, KLInterval *interval);

KLException *
KLIntervalIntersect(
    KLIntervalSet *interval_set, double *lower, double *upper,
    KLIntervalIntersectFun intersect_fun, void *client_data);

void
KLIntervalGetValue(const KLInterval *interval,
		 double **lower, double **upper, void **client_data);

void
KLIntervalGetString(
    KLIntervalSet *interval_set, KLArray *array);

void
KLIntervalTest(void);

#endif
