/*
 * $Id: kl_list.h,v 1.3 2000/10/06 00:08:19 laik Exp $
 *
 * This file contains interfaces for routines which manage lists.
 *
 * Copyright (C) 1993 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __LIST_H__
#define __LIST_H__

#include "kl_util.h"

typedef void (*KLListFreeFun)(void *client_data, void *item);

struct KLListNode;
typedef struct KLListNode KLListNode;

struct KLListNode {
    struct KLListNode *next;
    struct KLListNode *prev;
    void *item;
};

struct KLList;
typedef struct KLList KLList;

/*
 * The first field of a KLList must be a KLListNode.
 */
struct KLList {
    KLListNode nodes;
    unsigned long num_nodes;
};

#define KLListLoop(xx, zz) for ((xx) = KLListFirst(zz); !KLListFinished(zz, xx); \
				   (xx) = KLListNext(xx))

/* KLList creation and initialization. */
KLList *
KLListNew(void);
void
KLListVerify(KLList *list);

/* KLList freeing. */
void
KLListFreeItem(void *client_data, void *item);
void 
KLListFree(KLList *list, KLListFreeFun free_fun, void *client_data);
void
KLListFreeRange(KLList *list, KLListNode *begin_node, KLListNode *end_node, 
	      KLListFreeFun free_fun, void *client_data);
void
KLListFreeNode(KLList *list, KLListNode *node);

/* Add items to a list. */
KLListNode *
KLListInsertItem(KLList *list, KLListNode *after, void *item);
KLListNode *
KLListAppendItem(KLList *list, void *item);
KLListNode *
KLListPrependItem(KLList *list, void *item);

/* Add nodes to a list. */
void
KLListInsertList(KLList *dest, KLListNode *after, KLList *source);

/* Split some nodes from a list into a new list. */
KLList *
KLListSplitList(KLList *list, KLListNode *begin, KLListNode *end);

/* Testing the list code. */
void KLListTest(void);

inline extern unsigned long
KLListCount(KLList *list)
{
    return list->num_nodes;
}

inline extern KLListNode *
KLListFirst(const KLList *list)
{
    return list->nodes.next;
}

inline extern KLListNode *
KLListLast(const KLList *list)
{
    return list->nodes.prev;
}

inline extern void
KLListSetItem(KLListNode *node, void *item)
{
    node->item = item;
}

inline extern void *
KLListGetItem(const KLListNode *node)
{
    return node->item;
}

inline extern KLListNode *
KLListNext(const KLListNode *node)
{
    return node->next;
}

inline extern KLListNode *
KLListPrev(const KLListNode *node)
{
    return node->prev;
}

inline extern bool
KLListFinished(const KLList *list, const KLListNode *node)
{
    return ((const KLListNode *) list == node);
}

#endif
