/*
 * $Id: kl_statistics_functions.h,v 1.2 2000/10/05 22:05:56 laik Exp $
 *
 * Copyright (C) 2000 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __STATISTICS_FUNCTIONS_H__
#define __STATISTICS_FUNCTIONS_H__

//  Prototypes for probability functions from the cephes library

/* Combinatorial functions */
double factorial(int n);
double log_factorial(int n);
double choose(int n, int k);
double log_choose(int n, int k);

/* Binomial distribution */
double bdtr(int k, int n, double p);
double bdtrc(int k, int n, double p); /* complemented */
double bdtri(int k, int n, double y); /* inverse */

/* Negative binomial distribution */
double nbdtr(int k, int n, double p);
double nbdtri(int k, int n, double p);
double nbdtrc(int k, int n, double p); /* complemented */

/* Beta distribution */
double btdtr(double a, double b, double x); /* beta dist */
double incbet(double a, double b, double x); /* incomplete beta intgral */
double incbi(double a, double b, double p); /* inv incomplete beta integral */

/* Chi-square distribution */
double chdtr(double df, double x);
double chdtrc(double df, double x); /* complemented */
double chdtri(double df, double p); /* inverse of complemented */

/* F distribution */
double fdtr(int df1, int df2, double x);
double fdtrc(int df1, int df2, double x); /* complemented */
double fdtri(int df1, int df2, double p); /* inverse of complemented */

/* Gamma function */
double gammafn(double x);
double lgam(double x);  /* log of gamma function */
double igam(double a, double x); /* incomplete gamma integral */
double igamc(double a, double x); /* complemented */
double igami(double a, double p); /* inverse incomplete gamma integral */

/* Gamma distribution */
double gdtr(double a, double b, double x);
double gdtrc(double a, double b, double x); /* complemented */

/* Normal distribution */
double ndtr(double x);
double ndtri(double p); /* inverse */

/* Poisson distribution */
double pdtr(int k, double m);
double pdtrc(int k, double m); /* complemented */
double pdtri(int k, double p); /* inverse */

/* Student's t distribution */
double stdtr(int k, double t);
double stdtri(int k, double p); /* inverse */

/* Hypergeometric */
double hypergeometric_cdf(unsigned x, unsigned r, unsigned n, unsigned N);
unsigned inv_hypergeometric_cdf(double p, unsigned r, unsigned n, unsigned N);

/* Polynomial evaluation */
double polevl(double x, double coeff[], int);
double p1evl(double x, double coeff[], int);

#ifdef TEST

#endif

#endif
