/*
 * $Id: kl_statistics_kernel.h,v 1.3 2000/10/06 00:08:19 laik Exp $
 *
 * Copyright (C) 2000 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __STATISTICS_KERNEL_H__
#define __STATISTICS_KERNEL_H__

#include "kl_statistics_metric.h"
#include "kl_statistics_mapping.h"

/*
 * The different kinds of kernel functions.
 */
typedef enum KLStatisticsKernelFunction {
    KLStatisticsKernelFunctionUniform,        /* f(x) ~= 1 */
    KLStatisticsKernelFunctionTriangular,     /* f(x) ~= |1-x| */
} KLStatisticsKernelFunction;

typedef struct KLStatisticsKernelDef {
    unsigned int source_dim;                  /* Dimensions of source set */
    unsigned int mapped_dim;                  /* Dimensions of mapped set */
    const int *permutation;                   /* Select from source set */
    const double *resolution;                 /* Level to aggregate kernels */
    KLStatisticsKernelFunction kernel_function; /* Used to compute kernel */
    double kernel_bound_ratio;                /* Bounds radius of kernel */
} KLStatisticsKernelDef;

struct KLStatisticsKernel;
typedef struct KLStatisticsKernel KLStatisticsKernel;

KLStatisticsKernel *
KLStatisticsKernelNew(KLStatisticsKernelDef *kernel_def);

void
KLStatisticsKernelFree(KLStatisticsKernel *kernel);

KLStatisticsMetricType *
KLStatisticsKernelMetricTypeNew(KLStatisticsKernel *kernel);

KLStatisticsMapping *
KLStatisticsKernelMappingNew(KLStatisticsKernel *kernel);

#ifdef TEST

void
KLStatisticsKernelTest(void);

#endif

#endif
