/*
 * $Id: kl_statistics_mapping.h,v 1.3 2000/10/06 00:08:19 laik Exp $
 *
 * Copyright (C) 2000 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __STATISTICS_MAPPING_H__
#define __STATISTICS_MAPPING_H__

struct KLStatisticsMapping;
typedef struct KLStatisticsMapping KLStatisticsMapping;

/*
 * Generate the values of the mapped tuple from a source tuple.
 */
typedef void (KLStatisticsMappingMapFunc)(
    void *client_data, const double *source_values, double *mapped_lower,
    double *mapped_upper);

/*
 * Get the domain of the source tuples from the mapped values.
 */
typedef int (KLStatisticsMappingGetDomainFunc)(
    void *client_data, double *mapped_values, const double **source_lower, 
    const double **source_upper);

/*
 * Get the mapped range which could be affected by a change.
 */
typedef int (KLStatisticsMappingMergeChangeRangeFunc)(
    void *client_data, double *mapped_values, const double **mapped_lower, 
    const double **mapped_upper);

/*
 * Free client data.
 */
typedef void (KLStatisticsMappingFreeFunc)(void *client_data);

typedef struct KLStatisticsMappingDef {
    KLStatisticsMappingMapFunc *map;
    KLStatisticsMappingGetDomainFunc *get_domain;
    KLStatisticsMappingMergeChangeRangeFunc *merge_change_range;
    KLStatisticsMappingFreeFunc *free_fun;
    void *client_data;
} KLStatisticsMappingDef;

KLStatisticsMapping *
KLStatisticsMappingNew(KLStatisticsMappingDef *def);

void
KLStatisticsMappingFree(KLStatisticsMapping *sm);

void
KLStatisticsMappingMap(
    KLStatisticsMapping *sm, const double *source_values, double *mapped_lower,
    double *mapped_upper);

int
KLStatisticsMappingGetDomain(
    KLStatisticsMapping *sm, double *mapped_values, const double **source_lower, 
    const double **source_upper);

int
KLStatisticsMappingMergeChangeRange(
    KLStatisticsMapping *sm, double *mapped_values, const double **mapped_lower, 
    const double **mapped_upper);

#ifdef TEST

void
KLStatisticsMappingTest(void);

#endif

#endif
