/*
 * $Id: kl_util.h,v 1.7 2001/02/13 07:47:50 laik Exp $
 *
 *   This file contains some utility routines.
 *   Naming conventions:
 *   
 *   1) local variables and arguments: 
 *        lower case, underscore as delimiter
 *   2) static functions:
 *        lower case, underscore as delimiter
 *   3) global variables (ick), types
 *        capitalize first letter of each word, no delimiter,
 *        prepend defining file name
 *   4) external functions, macros
 *        capitalize first letter of each word, no delimiter
 *        prepend defining file name
 *   5) constants implemented as macros
 *        upper case, underscore as delimiter
 *
 * Copyright (C) 1993 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __UTIL_H__
#define __UTIL_H__

#include <stdlib.h>
#include <config.h>

#ifdef WITH_DMALLOC
#define DMALLOC_FUNC_CHECK
#include "dmalloc.h"
#endif

extern const char *KLUtilProgramName;

#undef min
#undef max
#ifdef __GNUC__
#define min(_x, _y) \
    ({ \
        typeof (_x) _x1 = (_x); \
        typeof (_y) _y1 = (_y); \
        _x1 < _y1 ? _x1 : _y1; \
    }) 

#define max(_x, _y) \
    ({ \
        typeof (_x) _x1 = (_x); \
        typeof (_y) _y1 = (_y); \
        _x1 > _y1 ? _x1 : _y1; \
    }) 
#else
#define min(_x, _y) _x1 < _y1 ? _x1 : _y1
#define max(_x, _y) x1 > _y1 ? _x1 : _y1
#endif

#if (!defined(CURSES))
#undef TRUE
#undef FALSE
typedef enum {
    FALSE = 0,
    TRUE = 1,
} bool;
#endif

unsigned long
KLUtilCRC(const unsigned char *blk_adr, unsigned long  blk_len);

unsigned long
KLUtilReverseBits(unsigned long l);

unsigned int
KLUtilHashString(const char *string);

#ifdef __GNUC__
#define KLUtilMalloc(_size)  \
    ({ \
        void *_rval3 = malloc(_size); \
        if (_rval3 == NULL) abort(); \
        _rval3; \
    })    
#else
void *
KLUtilMalloc(size_t size);
#endif

#endif
